"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeCypressPlugin = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const util_1 = require("util");
const debug_1 = __importDefault(require("debug"));
const debug = debug_1.default('cypress:vite-dev-server:plugin');
const read = util_1.promisify(fs_1.readFile);
const pluginName = 'cypress-transform-html';
const OSSepRE = new RegExp(`\\${path_1.sep}`, 'g');
function convertPathToPosix(path) {
    return path_1.sep === '/'
        ? path
        : path.replace(OSSepRE, '/');
}
const INIT_FILEPATH = path_1.resolve(__dirname, '../client/initCypressTests.js');
const HMR_DEPENDENCY_LOOKUP_MAX_ITERATION = 50;
function getSpecsPathsSet(specs) {
    return new Set(specs.map((spec) => spec.absolute));
}
const makeCypressPlugin = (projectRoot, supportFilePath, devServerEvents, specs) => {
    let base = '/';
    let specsPathsSet = getSpecsPathsSet(specs);
    devServerEvents.on('dev-server:specs:changed', (specs) => {
        specsPathsSet = getSpecsPathsSet(specs);
    });
    const posixSupportFilePath = supportFilePath ? convertPathToPosix(path_1.resolve(projectRoot, supportFilePath)) : undefined;
    const normalizedSupportFilePath = posixSupportFilePath ? `${base}@fs/${posixSupportFilePath}` : undefined;
    return {
        name: pluginName,
        enforce: 'pre',
        config(_, env) {
            if (env) {
                return {
                    define: {
                        'import.meta.env.__cypress_supportPath': JSON.stringify(normalizedSupportFilePath),
                        'import.meta.env.__cypress_originAutUrl': JSON.stringify(`__cypress/iframes/${convertPathToPosix(projectRoot)}/`),
                    },
                };
            }
        },
        configResolved(config) {
            base = config.base;
        },
        transformIndexHtml() {
            debug('transformIndexHtml with base', base);
            return [
                // load the script at the end of the body
                // script has to be loaded when the vite client is connected
                {
                    tag: 'script',
                    injectTo: 'body',
                    attrs: { type: 'module' },
                    children: `import(${JSON.stringify(`${base}@fs/${INIT_FILEPATH}`)})`,
                },
            ];
        },
        configureServer: async (server) => {
            const indexHtml = await read(path_1.resolve(__dirname, '..', 'index.html'), { encoding: 'utf8' });
            const transformedIndexHtml = await server.transformIndexHtml(base, indexHtml);
            server.middlewares.use(`${base}index.html`, (req, res) => res.end(transformedIndexHtml));
        },
        handleHotUpdate: ({ server, file }) => {
            debug('handleHotUpdate - file', file);
            // get the graph node for the file that just got updated
            let moduleImporters = server.moduleGraph.fileToModulesMap.get(file);
            let iterationNumber = 0;
            const exploredFiles = new Set();
            // until we reached a point where the current module is imported by no other
            while (moduleImporters?.size) {
                if (iterationNumber > HMR_DEPENDENCY_LOOKUP_MAX_ITERATION) {
                    debug(`max hmr iteration reached: ${HMR_DEPENDENCY_LOOKUP_MAX_ITERATION}; Rerun will not happen on this file change.`);
                    return [];
                }
                // as soon as we find one of the specs, we trigger the re-run of tests
                for (const mod of moduleImporters.values()) {
                    debug('handleHotUpdate - mod.file', mod.file);
                    if (mod.file === supportFilePath) {
                        debug('handleHotUpdate - support compile success');
                        devServerEvents.emit('dev-server:compile:success');
                        // if we update support we know we have to re-run it all
                        // no need to ckeck further
                        return [];
                    }
                    if (mod.file && specsPathsSet.has(mod.file)) {
                        debug('handleHotUpdate - spec compile success', mod.file);
                        devServerEvents.emit('dev-server:compile:success', { specFile: mod.file });
                        // if we find one spec, does not mean we are done yet,
                        // there could be other spec files to re-run
                        // see https://github.com/cypress-io/cypress/issues/17691
                    }
                    // to avoid circular updates, keep track of what we updated
                    if (mod.file) {
                        exploredFiles.add(mod.file);
                    }
                }
                // get all the modules that import the current one
                moduleImporters = getImporters(moduleImporters, exploredFiles);
                iterationNumber += 1;
            }
            return [];
        },
    };
};
exports.makeCypressPlugin = makeCypressPlugin;
function getImporters(modules, alreadyExploredFiles) {
    const allImporters = new Set();
    modules.forEach((m) => {
        if (m.file && !alreadyExploredFiles.has(m.file)) {
            m.importers.forEach((imp) => allImporters.add(imp));
        }
    });
    return allImporters;
}
